/****************************************************************************
** Meta object code from reading C++ file 'output.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/config/outputs/output.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'output.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN2bd6Config7Outputs6OutputE_t {};
} // unnamed namespace

template <> constexpr inline auto bd::Config::Outputs::Output::qt_create_metaobjectdata<qt_meta_tag_ZN2bd6Config7Outputs6OutputE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "bd::Config::Outputs::Output",
        "adaptiveSyncChanged",
        "",
        "uint32_t",
        "adaptiveSync",
        "disabledChanged",
        "disabled",
        "heightChanged",
        "height",
        "horizontalAnchorChanged",
        "bd::Outputs::Config::HorizontalAnchor::Type",
        "horizontalAnchor",
        "identifierChanged",
        "identifier",
        "primaryChanged",
        "primary",
        "refreshChanged",
        "refresh",
        "relativeOutputChanged",
        "relativeOutput",
        "scaleChanged",
        "scale",
        "transformChanged",
        "transform",
        "verticalAnchorChanged",
        "bd::Outputs::Config::VerticalAnchor::Type",
        "verticalAnchor",
        "widthChanged",
        "width",
        "xChanged",
        "x",
        "yChanged",
        "y"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'adaptiveSyncChanged'
        QtMocHelpers::SignalData<void(uint32_t)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'disabledChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Signal 'heightChanged'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Signal 'horizontalAnchorChanged'
        QtMocHelpers::SignalData<void(bd::Outputs::Config::HorizontalAnchor::Type)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'identifierChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'primaryChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Signal 'refreshChanged'
        QtMocHelpers::SignalData<void(qulonglong)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 17 },
        }}),
        // Signal 'relativeOutputChanged'
        QtMocHelpers::SignalData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Signal 'scaleChanged'
        QtMocHelpers::SignalData<void(qreal)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 21 },
        }}),
        // Signal 'transformChanged'
        QtMocHelpers::SignalData<void(quint16)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UShort, 23 },
        }}),
        // Signal 'verticalAnchorChanged'
        QtMocHelpers::SignalData<void(bd::Outputs::Config::VerticalAnchor::Type)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Signal 'widthChanged'
        QtMocHelpers::SignalData<void(int)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 28 },
        }}),
        // Signal 'xChanged'
        QtMocHelpers::SignalData<void(int)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 30 },
        }}),
        // Signal 'yChanged'
        QtMocHelpers::SignalData<void(int)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 32 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'identifier'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'width'
        QtMocHelpers::PropertyData<int>(28, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'height'
        QtMocHelpers::PropertyData<int>(8, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'refresh'
        QtMocHelpers::PropertyData<qulonglong>(17, QMetaType::ULongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'x'
        QtMocHelpers::PropertyData<int>(30, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'y'
        QtMocHelpers::PropertyData<int>(32, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'relativeOutput'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'horizontalAnchor'
        QtMocHelpers::PropertyData<bd::Outputs::Config::HorizontalAnchor::Type>(11, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'verticalAnchor'
        QtMocHelpers::PropertyData<bd::Outputs::Config::VerticalAnchor::Type>(26, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 10),
        // property 'transform'
        QtMocHelpers::PropertyData<quint16>(23, QMetaType::UShort, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'adaptiveSync'
        QtMocHelpers::PropertyData<uint32_t>(4, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'primary'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'scale'
        QtMocHelpers::PropertyData<qreal>(21, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'disabled'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Output, qt_meta_tag_ZN2bd6Config7Outputs6OutputE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN2bd6Config7Outputs6OutputE[] = {
    QMetaObject::SuperData::link<bd::Outputs::Config::HorizontalAnchor::staticMetaObject>(),
    QMetaObject::SuperData::link<bd::Outputs::Config::VerticalAnchor::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject bd::Config::Outputs::Output::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd6Config7Outputs6OutputE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd6Config7Outputs6OutputE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN2bd6Config7Outputs6OutputE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN2bd6Config7Outputs6OutputE_t>.metaTypes,
    nullptr
} };

void bd::Config::Outputs::Output::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Output *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->adaptiveSyncChanged((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 1: _t->disabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->heightChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->horizontalAnchorChanged((*reinterpret_cast<std::add_pointer_t<bd::Outputs::Config::HorizontalAnchor::Type>>(_a[1]))); break;
        case 4: _t->identifierChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->primaryChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->refreshChanged((*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[1]))); break;
        case 7: _t->relativeOutputChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->scaleChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 9: _t->transformChanged((*reinterpret_cast<std::add_pointer_t<quint16>>(_a[1]))); break;
        case 10: _t->verticalAnchorChanged((*reinterpret_cast<std::add_pointer_t<bd::Outputs::Config::VerticalAnchor::Type>>(_a[1]))); break;
        case 11: _t->widthChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->xChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->yChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Output::*)(uint32_t )>(_a, &Output::adaptiveSyncChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)(bool )>(_a, &Output::disabledChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)(int )>(_a, &Output::heightChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)(bd::Outputs::Config::HorizontalAnchor::Type )>(_a, &Output::horizontalAnchorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)(const QString & )>(_a, &Output::identifierChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)(bool )>(_a, &Output::primaryChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)(qulonglong )>(_a, &Output::refreshChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)(const QString & )>(_a, &Output::relativeOutputChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)(qreal )>(_a, &Output::scaleChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)(quint16 )>(_a, &Output::transformChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)(bd::Outputs::Config::VerticalAnchor::Type )>(_a, &Output::verticalAnchorChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)(int )>(_a, &Output::widthChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)(int )>(_a, &Output::xChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)(int )>(_a, &Output::yChanged, 13))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->identifier(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->width(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->height(); break;
        case 3: *reinterpret_cast<qulonglong*>(_v) = _t->refresh(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->x(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->y(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->relativeOutput(); break;
        case 7: *reinterpret_cast<bd::Outputs::Config::HorizontalAnchor::Type*>(_v) = _t->horizontalAnchor(); break;
        case 8: *reinterpret_cast<bd::Outputs::Config::VerticalAnchor::Type*>(_v) = _t->verticalAnchor(); break;
        case 9: *reinterpret_cast<quint16*>(_v) = _t->transform(); break;
        case 10: *reinterpret_cast<uint32_t*>(_v) = _t->adaptiveSync(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->primary(); break;
        case 12: *reinterpret_cast<qreal*>(_v) = _t->scale(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->disabled(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setIdentifier(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setWidth(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setHeight(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setRefresh(*reinterpret_cast<qulonglong*>(_v)); break;
        case 4: _t->setX(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setY(*reinterpret_cast<int*>(_v)); break;
        case 6: _t->setRelativeOutput(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setHorizontalAnchor(*reinterpret_cast<bd::Outputs::Config::HorizontalAnchor::Type*>(_v)); break;
        case 8: _t->setVerticalAnchor(*reinterpret_cast<bd::Outputs::Config::VerticalAnchor::Type*>(_v)); break;
        case 9: _t->setTransform(*reinterpret_cast<quint16*>(_v)); break;
        case 10: _t->setAdaptiveSync(*reinterpret_cast<uint32_t*>(_v)); break;
        case 11: _t->setPrimary(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setScale(*reinterpret_cast<qreal*>(_v)); break;
        case 13: _t->setDisabled(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *bd::Config::Outputs::Output::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *bd::Config::Outputs::Output::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd6Config7Outputs6OutputE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int bd::Config::Outputs::Output::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void bd::Config::Outputs::Output::adaptiveSyncChanged(uint32_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void bd::Config::Outputs::Output::disabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void bd::Config::Outputs::Output::heightChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void bd::Config::Outputs::Output::horizontalAnchorChanged(bd::Outputs::Config::HorizontalAnchor::Type _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void bd::Config::Outputs::Output::identifierChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void bd::Config::Outputs::Output::primaryChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void bd::Config::Outputs::Output::refreshChanged(qulonglong _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void bd::Config::Outputs::Output::relativeOutputChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void bd::Config::Outputs::Output::scaleChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void bd::Config::Outputs::Output::transformChanged(quint16 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void bd::Config::Outputs::Output::verticalAnchorChanged(bd::Outputs::Config::VerticalAnchor::Type _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void bd::Config::Outputs::Output::widthChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void bd::Config::Outputs::Output::xChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void bd::Config::Outputs::Output::yChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}
QT_WARNING_POP
