/****************************************************************************
** Meta object code from reading C++ file 'metamode.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/outputs/wlr/metamode.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'metamode.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN2bd7Outputs3Wlr8MetaModeE_t {};
} // unnamed namespace

template <> constexpr inline auto bd::Outputs::Wlr::MetaMode::qt_create_metaobjectdata<qt_meta_tag_ZN2bd7Outputs3Wlr8MetaModeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "bd::Outputs::Wlr::MetaMode",
        "D-Bus Interface",
        "org.buddiesofbudgie.Services.OutputMode",
        "availabilityChanged",
        "",
        "available",
        "done",
        "preferredChanged",
        "preferred",
        "refreshRateChanged",
        "refreshRate",
        "sizeChanged",
        "QSize",
        "size",
        "modeDisconnected",
        "setProperty",
        "MetaModeProperty::Property",
        "property",
        "QVariant",
        "value",
        "current",
        "height",
        "id",
        "width"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'availabilityChanged'
        QtMocHelpers::SignalData<void(bool)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'done'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'preferredChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Signal 'refreshRateChanged'
        QtMocHelpers::SignalData<void(qulonglong)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 10 },
        }}),
        // Signal 'sizeChanged'
        QtMocHelpers::SignalData<void(QSize)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'modeDisconnected'
        QtMocHelpers::SlotData<void()>(14, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setProperty'
        QtMocHelpers::SlotData<void(MetaModeProperty::Property, const QVariant &)>(15, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { 0x80000000 | 18, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'available'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'current'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'height'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'id'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'preferred'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'refreshRate'
        QtMocHelpers::PropertyData<qulonglong>(10, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'width'
        QtMocHelpers::PropertyData<int>(23, QMetaType::Int, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<MetaMode, qt_meta_tag_ZN2bd7Outputs3Wlr8MetaModeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject bd::Outputs::Wlr::MetaMode::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd7Outputs3Wlr8MetaModeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd7Outputs3Wlr8MetaModeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN2bd7Outputs3Wlr8MetaModeE_t>.metaTypes,
    nullptr
} };

void bd::Outputs::Wlr::MetaMode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MetaMode *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->availabilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->done(); break;
        case 2: _t->preferredChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->refreshRateChanged((*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[1]))); break;
        case 4: _t->sizeChanged((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 5: _t->modeDisconnected(); break;
        case 6: _t->setProperty((*reinterpret_cast<std::add_pointer_t<MetaModeProperty::Property>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MetaMode::*)(bool )>(_a, &MetaMode::availabilityChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaMode::*)()>(_a, &MetaMode::done, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaMode::*)(bool )>(_a, &MetaMode::preferredChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaMode::*)(qulonglong )>(_a, &MetaMode::refreshRateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaMode::*)(QSize )>(_a, &MetaMode::sizeChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->available(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->current(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->height(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->id(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->preferred(); break;
        case 5: *reinterpret_cast<qulonglong*>(_v) = _t->refreshRate(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->width(); break;
        default: break;
        }
    }
}

const QMetaObject *bd::Outputs::Wlr::MetaMode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *bd::Outputs::Wlr::MetaMode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd7Outputs3Wlr8MetaModeE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int bd::Outputs::Wlr::MetaMode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void bd::Outputs::Wlr::MetaMode::availabilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void bd::Outputs::Wlr::MetaMode::done()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void bd::Outputs::Wlr::MetaMode::preferredChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void bd::Outputs::Wlr::MetaMode::refreshRateChanged(qulonglong _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void bd::Outputs::Wlr::MetaMode::sizeChanged(QSize _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
