/****************************************************************************
** Meta object code from reading C++ file 'state.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/config/outputs/state.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'state.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN2bd6Config7Outputs5StateE_t {};
} // unnamed namespace

template <> constexpr inline auto bd::Config::Outputs::State::qt_create_metaobjectdata<qt_meta_tag_ZN2bd6Config7Outputs5StateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "bd::Config::Outputs::State",
        "activeGroupChanged",
        "",
        "QSharedPointer<Group>",
        "ActiveGroup",
        "matchingGroupChanged",
        "MatchingGroup",
        "saved",
        "apply",
        "deserialize",
        "save",
        "activeGroup",
        "matchingGroup",
        "groups",
        "QList<QSharedPointer<Group>>",
        "preferences",
        "QSharedPointer<GlobalPreferences>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activeGroupChanged'
        QtMocHelpers::SignalData<void(QSharedPointer<Group>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'matchingGroupChanged'
        QtMocHelpers::SignalData<void(QSharedPointer<Group>)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 },
        }}),
        // Signal 'saved'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'apply'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deserialize'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'save'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'activeGroup'
        QtMocHelpers::PropertyData<QSharedPointer<Group>>(11, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'matchingGroup'
        QtMocHelpers::PropertyData<QSharedPointer<Group>>(12, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'groups'
        QtMocHelpers::PropertyData<QList<QSharedPointer<Group>>>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'preferences'
        QtMocHelpers::PropertyData<QSharedPointer<GlobalPreferences>>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<State, qt_meta_tag_ZN2bd6Config7Outputs5StateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject bd::Config::Outputs::State::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd6Config7Outputs5StateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd6Config7Outputs5StateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN2bd6Config7Outputs5StateE_t>.metaTypes,
    nullptr
} };

void bd::Config::Outputs::State::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<State *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activeGroupChanged((*reinterpret_cast<std::add_pointer_t<QSharedPointer<Group>>>(_a[1]))); break;
        case 1: _t->matchingGroupChanged((*reinterpret_cast<std::add_pointer_t<QSharedPointer<Group>>>(_a[1]))); break;
        case 2: _t->saved(); break;
        case 3: _t->apply(); break;
        case 4: _t->deserialize(); break;
        case 5: _t->save(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<Group> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<Group> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (State::*)(QSharedPointer<Group> )>(_a, &State::activeGroupChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (State::*)(QSharedPointer<Group> )>(_a, &State::matchingGroupChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (State::*)()>(_a, &State::saved, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QSharedPointer<Group>> >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSharedPointer<GlobalPreferences> >(); break;
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSharedPointer<Group> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QSharedPointer<Group>*>(_v) = _t->activeGroup(); break;
        case 1: *reinterpret_cast<QSharedPointer<Group>*>(_v) = _t->matchingGroup(); break;
        case 2: *reinterpret_cast<QList<QSharedPointer<Group>>*>(_v) = _t->groups(); break;
        case 3: *reinterpret_cast<QSharedPointer<GlobalPreferences>*>(_v) = _t->preferences(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setActiveGroup(*reinterpret_cast<QSharedPointer<Group>*>(_v)); break;
        case 1: _t->setMatchingGroup(*reinterpret_cast<QSharedPointer<Group>*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *bd::Config::Outputs::State::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *bd::Config::Outputs::State::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd6Config7Outputs5StateE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int bd::Config::Outputs::State::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void bd::Config::Outputs::State::activeGroupChanged(QSharedPointer<Group> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void bd::Config::Outputs::State::matchingGroupChanged(QSharedPointer<Group> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void bd::Config::Outputs::State::saved()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
