// This file was generated by qtwaylandscanner
// source file is wlr-output-management-unstable-v1.xml

#include "qwayland-wlr-output-management-unstable-v1.h"

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {

static inline void *wlRegistryBind(struct ::wl_registry *registry, uint32_t name, const struct ::wl_interface *interface, uint32_t version)
{
    const uint32_t bindOpCode = 0;
    return (void *) wl_proxy_marshal_constructor_versioned((struct wl_proxy *) registry,
    bindOpCode, interface, version, name, interface->name, version, nullptr);
}

    zwlr_output_manager_v1::zwlr_output_manager_v1(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    zwlr_output_manager_v1::zwlr_output_manager_v1(struct ::zwlr_output_manager_v1 *obj)
        : m_zwlr_output_manager_v1(obj)
    {
        init_listener();
    }

    zwlr_output_manager_v1::zwlr_output_manager_v1()
        : m_zwlr_output_manager_v1(nullptr)
    {
    }

    zwlr_output_manager_v1::~zwlr_output_manager_v1()
    {
    }

    void zwlr_output_manager_v1::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_zwlr_output_manager_v1 = static_cast<struct ::zwlr_output_manager_v1 *>(wlRegistryBind(registry, id, &zwlr_output_manager_v1_interface, version));
        init_listener();
    }

    void zwlr_output_manager_v1::init(struct ::zwlr_output_manager_v1 *obj)
    {
        m_zwlr_output_manager_v1 = obj;
        init_listener();
    }

    zwlr_output_manager_v1 *zwlr_output_manager_v1::fromObject(struct ::zwlr_output_manager_v1 *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_zwlr_output_manager_v1_listener)
            return nullptr;
        return static_cast<zwlr_output_manager_v1 *>(zwlr_output_manager_v1_get_user_data(object));
    }

    bool zwlr_output_manager_v1::isInitialized() const
    {
        return m_zwlr_output_manager_v1 != nullptr;
    }

    uint32_t zwlr_output_manager_v1::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_zwlr_output_manager_v1));
    }

    const struct wl_interface *zwlr_output_manager_v1::interface()
    {
        return &::zwlr_output_manager_v1_interface;
    }

    struct ::zwlr_output_configuration_v1 *zwlr_output_manager_v1::create_configuration(uint32_t serial)
    {
        struct ::zwlr_output_configuration_v1 * id = ::zwlr_output_manager_v1_create_configuration(
            m_zwlr_output_manager_v1,
            serial);
        return id;
    }

    void zwlr_output_manager_v1::stop()
    {
        ::zwlr_output_manager_v1_stop(
            m_zwlr_output_manager_v1);
    }

    void zwlr_output_manager_v1::zwlr_output_manager_v1_head(struct ::zwlr_output_head_v1 *)
    {
    }

    void zwlr_output_manager_v1::handle_head(
        void *data,
        struct ::zwlr_output_manager_v1 *object,
        struct ::zwlr_output_head_v1 *head)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_manager_v1 *>(data)->zwlr_output_manager_v1_head(
            head);
    }

    void zwlr_output_manager_v1::zwlr_output_manager_v1_done(uint32_t )
    {
    }

    void zwlr_output_manager_v1::handle_done(
        void *data,
        struct ::zwlr_output_manager_v1 *object,
        uint32_t serial)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_manager_v1 *>(data)->zwlr_output_manager_v1_done(
            serial);
    }

    void zwlr_output_manager_v1::zwlr_output_manager_v1_finished()
    {
    }

    void zwlr_output_manager_v1::handle_finished(
        void *data,
        struct ::zwlr_output_manager_v1 *object)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_manager_v1 *>(data)->zwlr_output_manager_v1_finished();
    }

    const struct zwlr_output_manager_v1_listener zwlr_output_manager_v1::m_zwlr_output_manager_v1_listener = {
        zwlr_output_manager_v1::handle_head,
        zwlr_output_manager_v1::handle_done,
        zwlr_output_manager_v1::handle_finished,
    };

    void zwlr_output_manager_v1::init_listener()
    {
        zwlr_output_manager_v1_add_listener(m_zwlr_output_manager_v1, &m_zwlr_output_manager_v1_listener, this);
    }

    zwlr_output_head_v1::zwlr_output_head_v1(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    zwlr_output_head_v1::zwlr_output_head_v1(struct ::zwlr_output_head_v1 *obj)
        : m_zwlr_output_head_v1(obj)
    {
        init_listener();
    }

    zwlr_output_head_v1::zwlr_output_head_v1()
        : m_zwlr_output_head_v1(nullptr)
    {
    }

    zwlr_output_head_v1::~zwlr_output_head_v1()
    {
    }

    void zwlr_output_head_v1::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_zwlr_output_head_v1 = static_cast<struct ::zwlr_output_head_v1 *>(wlRegistryBind(registry, id, &zwlr_output_head_v1_interface, version));
        init_listener();
    }

    void zwlr_output_head_v1::init(struct ::zwlr_output_head_v1 *obj)
    {
        m_zwlr_output_head_v1 = obj;
        init_listener();
    }

    zwlr_output_head_v1 *zwlr_output_head_v1::fromObject(struct ::zwlr_output_head_v1 *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_zwlr_output_head_v1_listener)
            return nullptr;
        return static_cast<zwlr_output_head_v1 *>(zwlr_output_head_v1_get_user_data(object));
    }

    bool zwlr_output_head_v1::isInitialized() const
    {
        return m_zwlr_output_head_v1 != nullptr;
    }

    uint32_t zwlr_output_head_v1::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_zwlr_output_head_v1));
    }

    const struct wl_interface *zwlr_output_head_v1::interface()
    {
        return &::zwlr_output_head_v1_interface;
    }

    void zwlr_output_head_v1::release()
    {
        ::zwlr_output_head_v1_release(
            m_zwlr_output_head_v1);
        m_zwlr_output_head_v1 = nullptr;
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_name(const QString &)
    {
    }

    void zwlr_output_head_v1::handle_name(
        void *data,
        struct ::zwlr_output_head_v1 *object,
        const char *name)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_name(
            QString::fromUtf8(name));
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_description(const QString &)
    {
    }

    void zwlr_output_head_v1::handle_description(
        void *data,
        struct ::zwlr_output_head_v1 *object,
        const char *description)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_description(
            QString::fromUtf8(description));
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_physical_size(int32_t , int32_t )
    {
    }

    void zwlr_output_head_v1::handle_physical_size(
        void *data,
        struct ::zwlr_output_head_v1 *object,
        int32_t width,
        int32_t height)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_physical_size(
            width,
            height);
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_mode(struct ::zwlr_output_mode_v1 *)
    {
    }

    void zwlr_output_head_v1::handle_mode(
        void *data,
        struct ::zwlr_output_head_v1 *object,
        struct ::zwlr_output_mode_v1 *mode)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_mode(
            mode);
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_enabled(int32_t )
    {
    }

    void zwlr_output_head_v1::handle_enabled(
        void *data,
        struct ::zwlr_output_head_v1 *object,
        int32_t enabled)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_enabled(
            enabled);
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_current_mode(struct ::zwlr_output_mode_v1 *)
    {
    }

    void zwlr_output_head_v1::handle_current_mode(
        void *data,
        struct ::zwlr_output_head_v1 *object,
        struct ::zwlr_output_mode_v1 *mode)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_current_mode(
            mode);
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_position(int32_t , int32_t )
    {
    }

    void zwlr_output_head_v1::handle_position(
        void *data,
        struct ::zwlr_output_head_v1 *object,
        int32_t x,
        int32_t y)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_position(
            x,
            y);
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_transform(int32_t )
    {
    }

    void zwlr_output_head_v1::handle_transform(
        void *data,
        struct ::zwlr_output_head_v1 *object,
        int32_t transform)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_transform(
            transform);
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_scale(wl_fixed_t )
    {
    }

    void zwlr_output_head_v1::handle_scale(
        void *data,
        struct ::zwlr_output_head_v1 *object,
        wl_fixed_t scale)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_scale(
            scale);
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_finished()
    {
    }

    void zwlr_output_head_v1::handle_finished(
        void *data,
        struct ::zwlr_output_head_v1 *object)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_finished();
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_make(const QString &)
    {
    }

    void zwlr_output_head_v1::handle_make(
        void *data,
        struct ::zwlr_output_head_v1 *object,
        const char *make)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_make(
            QString::fromUtf8(make));
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_model(const QString &)
    {
    }

    void zwlr_output_head_v1::handle_model(
        void *data,
        struct ::zwlr_output_head_v1 *object,
        const char *model)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_model(
            QString::fromUtf8(model));
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_serial_number(const QString &)
    {
    }

    void zwlr_output_head_v1::handle_serial_number(
        void *data,
        struct ::zwlr_output_head_v1 *object,
        const char *serial_number)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_serial_number(
            QString::fromUtf8(serial_number));
    }

    void zwlr_output_head_v1::zwlr_output_head_v1_adaptive_sync(uint32_t )
    {
    }

    void zwlr_output_head_v1::handle_adaptive_sync(
        void *data,
        struct ::zwlr_output_head_v1 *object,
        uint32_t state)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_head_v1 *>(data)->zwlr_output_head_v1_adaptive_sync(
            state);
    }

    const struct zwlr_output_head_v1_listener zwlr_output_head_v1::m_zwlr_output_head_v1_listener = {
        zwlr_output_head_v1::handle_name,
        zwlr_output_head_v1::handle_description,
        zwlr_output_head_v1::handle_physical_size,
        zwlr_output_head_v1::handle_mode,
        zwlr_output_head_v1::handle_enabled,
        zwlr_output_head_v1::handle_current_mode,
        zwlr_output_head_v1::handle_position,
        zwlr_output_head_v1::handle_transform,
        zwlr_output_head_v1::handle_scale,
        zwlr_output_head_v1::handle_finished,
        zwlr_output_head_v1::handle_make,
        zwlr_output_head_v1::handle_model,
        zwlr_output_head_v1::handle_serial_number,
        zwlr_output_head_v1::handle_adaptive_sync,
    };

    void zwlr_output_head_v1::init_listener()
    {
        zwlr_output_head_v1_add_listener(m_zwlr_output_head_v1, &m_zwlr_output_head_v1_listener, this);
    }

    zwlr_output_mode_v1::zwlr_output_mode_v1(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    zwlr_output_mode_v1::zwlr_output_mode_v1(struct ::zwlr_output_mode_v1 *obj)
        : m_zwlr_output_mode_v1(obj)
    {
        init_listener();
    }

    zwlr_output_mode_v1::zwlr_output_mode_v1()
        : m_zwlr_output_mode_v1(nullptr)
    {
    }

    zwlr_output_mode_v1::~zwlr_output_mode_v1()
    {
    }

    void zwlr_output_mode_v1::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_zwlr_output_mode_v1 = static_cast<struct ::zwlr_output_mode_v1 *>(wlRegistryBind(registry, id, &zwlr_output_mode_v1_interface, version));
        init_listener();
    }

    void zwlr_output_mode_v1::init(struct ::zwlr_output_mode_v1 *obj)
    {
        m_zwlr_output_mode_v1 = obj;
        init_listener();
    }

    zwlr_output_mode_v1 *zwlr_output_mode_v1::fromObject(struct ::zwlr_output_mode_v1 *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_zwlr_output_mode_v1_listener)
            return nullptr;
        return static_cast<zwlr_output_mode_v1 *>(zwlr_output_mode_v1_get_user_data(object));
    }

    bool zwlr_output_mode_v1::isInitialized() const
    {
        return m_zwlr_output_mode_v1 != nullptr;
    }

    uint32_t zwlr_output_mode_v1::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_zwlr_output_mode_v1));
    }

    const struct wl_interface *zwlr_output_mode_v1::interface()
    {
        return &::zwlr_output_mode_v1_interface;
    }

    void zwlr_output_mode_v1::release()
    {
        ::zwlr_output_mode_v1_release(
            m_zwlr_output_mode_v1);
        m_zwlr_output_mode_v1 = nullptr;
    }

    void zwlr_output_mode_v1::zwlr_output_mode_v1_size(int32_t , int32_t )
    {
    }

    void zwlr_output_mode_v1::handle_size(
        void *data,
        struct ::zwlr_output_mode_v1 *object,
        int32_t width,
        int32_t height)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_mode_v1 *>(data)->zwlr_output_mode_v1_size(
            width,
            height);
    }

    void zwlr_output_mode_v1::zwlr_output_mode_v1_refresh(int32_t )
    {
    }

    void zwlr_output_mode_v1::handle_refresh(
        void *data,
        struct ::zwlr_output_mode_v1 *object,
        int32_t refresh)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_mode_v1 *>(data)->zwlr_output_mode_v1_refresh(
            refresh);
    }

    void zwlr_output_mode_v1::zwlr_output_mode_v1_preferred()
    {
    }

    void zwlr_output_mode_v1::handle_preferred(
        void *data,
        struct ::zwlr_output_mode_v1 *object)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_mode_v1 *>(data)->zwlr_output_mode_v1_preferred();
    }

    void zwlr_output_mode_v1::zwlr_output_mode_v1_finished()
    {
    }

    void zwlr_output_mode_v1::handle_finished(
        void *data,
        struct ::zwlr_output_mode_v1 *object)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_mode_v1 *>(data)->zwlr_output_mode_v1_finished();
    }

    const struct zwlr_output_mode_v1_listener zwlr_output_mode_v1::m_zwlr_output_mode_v1_listener = {
        zwlr_output_mode_v1::handle_size,
        zwlr_output_mode_v1::handle_refresh,
        zwlr_output_mode_v1::handle_preferred,
        zwlr_output_mode_v1::handle_finished,
    };

    void zwlr_output_mode_v1::init_listener()
    {
        zwlr_output_mode_v1_add_listener(m_zwlr_output_mode_v1, &m_zwlr_output_mode_v1_listener, this);
    }

    zwlr_output_configuration_v1::zwlr_output_configuration_v1(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    zwlr_output_configuration_v1::zwlr_output_configuration_v1(struct ::zwlr_output_configuration_v1 *obj)
        : m_zwlr_output_configuration_v1(obj)
    {
        init_listener();
    }

    zwlr_output_configuration_v1::zwlr_output_configuration_v1()
        : m_zwlr_output_configuration_v1(nullptr)
    {
    }

    zwlr_output_configuration_v1::~zwlr_output_configuration_v1()
    {
    }

    void zwlr_output_configuration_v1::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_zwlr_output_configuration_v1 = static_cast<struct ::zwlr_output_configuration_v1 *>(wlRegistryBind(registry, id, &zwlr_output_configuration_v1_interface, version));
        init_listener();
    }

    void zwlr_output_configuration_v1::init(struct ::zwlr_output_configuration_v1 *obj)
    {
        m_zwlr_output_configuration_v1 = obj;
        init_listener();
    }

    zwlr_output_configuration_v1 *zwlr_output_configuration_v1::fromObject(struct ::zwlr_output_configuration_v1 *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_zwlr_output_configuration_v1_listener)
            return nullptr;
        return static_cast<zwlr_output_configuration_v1 *>(zwlr_output_configuration_v1_get_user_data(object));
    }

    bool zwlr_output_configuration_v1::isInitialized() const
    {
        return m_zwlr_output_configuration_v1 != nullptr;
    }

    uint32_t zwlr_output_configuration_v1::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_zwlr_output_configuration_v1));
    }

    const struct wl_interface *zwlr_output_configuration_v1::interface()
    {
        return &::zwlr_output_configuration_v1_interface;
    }

    struct ::zwlr_output_configuration_head_v1 *zwlr_output_configuration_v1::enable_head(struct ::zwlr_output_head_v1 *head)
    {
        struct ::zwlr_output_configuration_head_v1 * id = ::zwlr_output_configuration_v1_enable_head(
            m_zwlr_output_configuration_v1,
            head);
        return id;
    }

    void zwlr_output_configuration_v1::disable_head(struct ::zwlr_output_head_v1 *head)
    {
        ::zwlr_output_configuration_v1_disable_head(
            m_zwlr_output_configuration_v1,
            head);
    }

    void zwlr_output_configuration_v1::apply()
    {
        ::zwlr_output_configuration_v1_apply(
            m_zwlr_output_configuration_v1);
    }

    void zwlr_output_configuration_v1::test()
    {
        ::zwlr_output_configuration_v1_test(
            m_zwlr_output_configuration_v1);
    }

    void zwlr_output_configuration_v1::destroy()
    {
        ::zwlr_output_configuration_v1_destroy(
            m_zwlr_output_configuration_v1);
        m_zwlr_output_configuration_v1 = nullptr;
    }

    void zwlr_output_configuration_v1::zwlr_output_configuration_v1_succeeded()
    {
    }

    void zwlr_output_configuration_v1::handle_succeeded(
        void *data,
        struct ::zwlr_output_configuration_v1 *object)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_configuration_v1 *>(data)->zwlr_output_configuration_v1_succeeded();
    }

    void zwlr_output_configuration_v1::zwlr_output_configuration_v1_failed()
    {
    }

    void zwlr_output_configuration_v1::handle_failed(
        void *data,
        struct ::zwlr_output_configuration_v1 *object)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_configuration_v1 *>(data)->zwlr_output_configuration_v1_failed();
    }

    void zwlr_output_configuration_v1::zwlr_output_configuration_v1_cancelled()
    {
    }

    void zwlr_output_configuration_v1::handle_cancelled(
        void *data,
        struct ::zwlr_output_configuration_v1 *object)
    {
        Q_UNUSED(object);
        static_cast<zwlr_output_configuration_v1 *>(data)->zwlr_output_configuration_v1_cancelled();
    }

    const struct zwlr_output_configuration_v1_listener zwlr_output_configuration_v1::m_zwlr_output_configuration_v1_listener = {
        zwlr_output_configuration_v1::handle_succeeded,
        zwlr_output_configuration_v1::handle_failed,
        zwlr_output_configuration_v1::handle_cancelled,
    };

    void zwlr_output_configuration_v1::init_listener()
    {
        zwlr_output_configuration_v1_add_listener(m_zwlr_output_configuration_v1, &m_zwlr_output_configuration_v1_listener, this);
    }

    zwlr_output_configuration_head_v1::zwlr_output_configuration_head_v1(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    zwlr_output_configuration_head_v1::zwlr_output_configuration_head_v1(struct ::zwlr_output_configuration_head_v1 *obj)
        : m_zwlr_output_configuration_head_v1(obj)
    {
    }

    zwlr_output_configuration_head_v1::zwlr_output_configuration_head_v1()
        : m_zwlr_output_configuration_head_v1(nullptr)
    {
    }

    zwlr_output_configuration_head_v1::~zwlr_output_configuration_head_v1()
    {
    }

    void zwlr_output_configuration_head_v1::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_zwlr_output_configuration_head_v1 = static_cast<struct ::zwlr_output_configuration_head_v1 *>(wlRegistryBind(registry, id, &zwlr_output_configuration_head_v1_interface, version));
    }

    void zwlr_output_configuration_head_v1::init(struct ::zwlr_output_configuration_head_v1 *obj)
    {
        m_zwlr_output_configuration_head_v1 = obj;
    }

    zwlr_output_configuration_head_v1 *zwlr_output_configuration_head_v1::fromObject(struct ::zwlr_output_configuration_head_v1 *object)
    {
        return static_cast<zwlr_output_configuration_head_v1 *>(zwlr_output_configuration_head_v1_get_user_data(object));
    }

    bool zwlr_output_configuration_head_v1::isInitialized() const
    {
        return m_zwlr_output_configuration_head_v1 != nullptr;
    }

    uint32_t zwlr_output_configuration_head_v1::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_zwlr_output_configuration_head_v1));
    }

    const struct wl_interface *zwlr_output_configuration_head_v1::interface()
    {
        return &::zwlr_output_configuration_head_v1_interface;
    }

    void zwlr_output_configuration_head_v1::set_mode(struct ::zwlr_output_mode_v1 *mode)
    {
        ::zwlr_output_configuration_head_v1_set_mode(
            m_zwlr_output_configuration_head_v1,
            mode);
    }

    void zwlr_output_configuration_head_v1::set_custom_mode(int32_t width, int32_t height, int32_t refresh)
    {
        ::zwlr_output_configuration_head_v1_set_custom_mode(
            m_zwlr_output_configuration_head_v1,
            width,
            height,
            refresh);
    }

    void zwlr_output_configuration_head_v1::set_position(int32_t x, int32_t y)
    {
        ::zwlr_output_configuration_head_v1_set_position(
            m_zwlr_output_configuration_head_v1,
            x,
            y);
    }

    void zwlr_output_configuration_head_v1::set_transform(int32_t transform)
    {
        ::zwlr_output_configuration_head_v1_set_transform(
            m_zwlr_output_configuration_head_v1,
            transform);
    }

    void zwlr_output_configuration_head_v1::set_scale(wl_fixed_t scale)
    {
        ::zwlr_output_configuration_head_v1_set_scale(
            m_zwlr_output_configuration_head_v1,
            scale);
    }

    void zwlr_output_configuration_head_v1::set_adaptive_sync(uint32_t state)
    {
        ::zwlr_output_configuration_head_v1_set_adaptive_sync(
            m_zwlr_output_configuration_head_v1,
            state);
    }
}

QT_WARNING_POP
QT_END_NAMESPACE
