/****************************************************************************
** Meta object code from reading C++ file 'metahead.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/outputs/wlr/metahead.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'metahead.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN2bd7Outputs3Wlr8MetaHeadE_t {};
} // unnamed namespace

template <> constexpr inline auto bd::Outputs::Wlr::MetaHead::qt_create_metaobjectdata<qt_meta_tag_ZN2bd7Outputs3Wlr8MetaHeadE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "bd::Outputs::Wlr::MetaHead",
        "D-Bus Interface",
        "org.buddiesofbudgie.Services.Output",
        "headAvailable",
        "",
        "headNoLongerAvailable",
        "stateChanged",
        "adaptiveSyncChanged",
        "adaptiveSync",
        "currentModeChanged",
        "bd::Outputs::OutputModeInfo",
        "currentMode",
        "descriptionChanged",
        "description",
        "enabledChanged",
        "enabled",
        "heightChanged",
        "height",
        "horizontalAnchorChanged",
        "horizontalAnchor",
        "makeChanged",
        "make",
        "mirrorOfChanged",
        "mirrorOf",
        "modelChanged",
        "model",
        "modesChanged",
        "nameChanged",
        "name",
        "positionChanged",
        "QPoint",
        "position",
        "primaryChanged",
        "primary",
        "refreshRateChanged",
        "refreshRate",
        "relativeToChanged",
        "relativeTo",
        "scaleChanged",
        "scale",
        "serialChanged",
        "serial",
        "transformChanged",
        "transform",
        "verticalAnchorChanged",
        "verticalAnchor",
        "widthChanged",
        "width",
        "xChanged",
        "x",
        "yChanged",
        "y",
        "addMode",
        "QSharedPointer<bd::Outputs::Wlr::MetaMode>",
        "::zwlr_output_mode_v1*",
        "mode",
        "currentZwlrModeChanged",
        "headDisconnected",
        "setProperty",
        "MetaHeadProperty::Property",
        "property",
        "QVariant",
        "value",
        "builtIn",
        "modes",
        "bd::Outputs::OutputModesMap"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'headAvailable'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'headNoLongerAvailable'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'adaptiveSyncChanged'
        QtMocHelpers::SignalData<void(uint)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 8 },
        }}),
        // Signal 'currentModeChanged'
        QtMocHelpers::SignalData<void(const bd::Outputs::OutputModeInfo &)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'descriptionChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'enabledChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Signal 'heightChanged'
        QtMocHelpers::SignalData<void(int)>(16, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
        // Signal 'horizontalAnchorChanged'
        QtMocHelpers::SignalData<void(const QString &)>(18, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Signal 'makeChanged'
        QtMocHelpers::SignalData<void(const QString &)>(20, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Signal 'mirrorOfChanged'
        QtMocHelpers::SignalData<void(const QString &)>(22, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Signal 'modelChanged'
        QtMocHelpers::SignalData<void(const QString &)>(24, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Signal 'modesChanged'
        QtMocHelpers::SignalData<void()>(26, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(27, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 28 },
        }}),
        // Signal 'positionChanged'
        QtMocHelpers::SignalData<void(const QPoint &)>(29, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Signal 'primaryChanged'
        QtMocHelpers::SignalData<void(bool)>(32, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 33 },
        }}),
        // Signal 'refreshRateChanged'
        QtMocHelpers::SignalData<void(qulonglong)>(34, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 35 },
        }}),
        // Signal 'relativeToChanged'
        QtMocHelpers::SignalData<void(const QString &)>(36, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 37 },
        }}),
        // Signal 'scaleChanged'
        QtMocHelpers::SignalData<void(double)>(38, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 39 },
        }}),
        // Signal 'serialChanged'
        QtMocHelpers::SignalData<void(const QString &)>(40, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 41 },
        }}),
        // Signal 'transformChanged'
        QtMocHelpers::SignalData<void(quint16)>(42, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UShort, 43 },
        }}),
        // Signal 'verticalAnchorChanged'
        QtMocHelpers::SignalData<void(const QString &)>(44, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 45 },
        }}),
        // Signal 'widthChanged'
        QtMocHelpers::SignalData<void(int)>(46, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 47 },
        }}),
        // Signal 'xChanged'
        QtMocHelpers::SignalData<void(int)>(48, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 49 },
        }}),
        // Signal 'yChanged'
        QtMocHelpers::SignalData<void(int)>(50, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 51 },
        }}),
        // Slot 'addMode'
        QtMocHelpers::SlotData<QSharedPointer<bd::Outputs::Wlr::MetaMode>(::zwlr_output_mode_v1 *)>(52, 4, QMC::AccessPrivate, 0x80000000 | 53, {{
            { 0x80000000 | 54, 55 },
        }}),
        // Slot 'currentZwlrModeChanged'
        QtMocHelpers::SlotData<void(::zwlr_output_mode_v1 *)>(56, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 54, 55 },
        }}),
        // Slot 'headDisconnected'
        QtMocHelpers::SlotData<void()>(57, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setProperty'
        QtMocHelpers::SlotData<void(MetaHeadProperty::Property, const QVariant &)>(58, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 59, 60 }, { 0x80000000 | 61, 62 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'adaptiveSync'
        QtMocHelpers::PropertyData<uint>(8, QMetaType::UInt, QMC::DefaultPropertyFlags, 3),
        // property 'builtIn'
        QtMocHelpers::PropertyData<bool>(63, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'currentMode'
        QtMocHelpers::PropertyData<bd::Outputs::OutputModeInfo>(11, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'description'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'enabled'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
        // property 'height'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags, 7),
        // property 'horizontalAnchor'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags, 8),
        // property 'make'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags, 9),
        // property 'modes'
        QtMocHelpers::PropertyData<bd::Outputs::OutputModesMap>(64, 0x80000000 | 65, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 12),
        // property 'mirrorOf'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags, 10),
        // property 'model'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags, 11),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags, 13),
        // property 'primary'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags, 15),
        // property 'refreshRate'
        QtMocHelpers::PropertyData<qulonglong>(35, QMetaType::ULongLong, QMC::DefaultPropertyFlags, 16),
        // property 'relativeTo'
        QtMocHelpers::PropertyData<QString>(37, QMetaType::QString, QMC::DefaultPropertyFlags, 17),
        // property 'scale'
        QtMocHelpers::PropertyData<double>(39, QMetaType::Double, QMC::DefaultPropertyFlags, 18),
        // property 'serial'
        QtMocHelpers::PropertyData<QString>(41, QMetaType::QString, QMC::DefaultPropertyFlags, 19),
        // property 'transform'
        QtMocHelpers::PropertyData<quint16>(43, QMetaType::UShort, QMC::DefaultPropertyFlags, 20),
        // property 'verticalAnchor'
        QtMocHelpers::PropertyData<QString>(45, QMetaType::QString, QMC::DefaultPropertyFlags, 21),
        // property 'width'
        QtMocHelpers::PropertyData<int>(47, QMetaType::Int, QMC::DefaultPropertyFlags, 22),
        // property 'x'
        QtMocHelpers::PropertyData<int>(49, QMetaType::Int, QMC::DefaultPropertyFlags, 23),
        // property 'y'
        QtMocHelpers::PropertyData<int>(51, QMetaType::Int, QMC::DefaultPropertyFlags, 24),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<MetaHead, qt_meta_tag_ZN2bd7Outputs3Wlr8MetaHeadE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject bd::Outputs::Wlr::MetaHead::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd7Outputs3Wlr8MetaHeadE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd7Outputs3Wlr8MetaHeadE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN2bd7Outputs3Wlr8MetaHeadE_t>.metaTypes,
    nullptr
} };

void bd::Outputs::Wlr::MetaHead::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MetaHead *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->headAvailable(); break;
        case 1: _t->headNoLongerAvailable(); break;
        case 2: _t->stateChanged(); break;
        case 3: _t->adaptiveSyncChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 4: _t->currentModeChanged((*reinterpret_cast<std::add_pointer_t<bd::Outputs::OutputModeInfo>>(_a[1]))); break;
        case 5: _t->descriptionChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->enabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->heightChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->horizontalAnchorChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->makeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->mirrorOfChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->modelChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->modesChanged(); break;
        case 13: _t->nameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->positionChanged((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 15: _t->primaryChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->refreshRateChanged((*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[1]))); break;
        case 17: _t->relativeToChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->scaleChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->serialChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->transformChanged((*reinterpret_cast<std::add_pointer_t<quint16>>(_a[1]))); break;
        case 21: _t->verticalAnchorChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->widthChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->xChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->yChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: { QSharedPointer<bd::Outputs::Wlr::MetaMode> _r = _t->addMode((*reinterpret_cast<std::add_pointer_t<::zwlr_output_mode_v1*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QSharedPointer<bd::Outputs::Wlr::MetaMode>*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->currentZwlrModeChanged((*reinterpret_cast<std::add_pointer_t<::zwlr_output_mode_v1*>>(_a[1]))); break;
        case 27: _t->headDisconnected(); break;
        case 28: _t->setProperty((*reinterpret_cast<std::add_pointer_t<MetaHeadProperty::Property>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< bd::Outputs::OutputModeInfo >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)()>(_a, &MetaHead::headAvailable, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)()>(_a, &MetaHead::headNoLongerAvailable, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)()>(_a, &MetaHead::stateChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(uint )>(_a, &MetaHead::adaptiveSyncChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(const bd::Outputs::OutputModeInfo & )>(_a, &MetaHead::currentModeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(const QString & )>(_a, &MetaHead::descriptionChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(bool )>(_a, &MetaHead::enabledChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(int )>(_a, &MetaHead::heightChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(const QString & )>(_a, &MetaHead::horizontalAnchorChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(const QString & )>(_a, &MetaHead::makeChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(const QString & )>(_a, &MetaHead::mirrorOfChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(const QString & )>(_a, &MetaHead::modelChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)()>(_a, &MetaHead::modesChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(const QString & )>(_a, &MetaHead::nameChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(const QPoint & )>(_a, &MetaHead::positionChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(bool )>(_a, &MetaHead::primaryChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(qulonglong )>(_a, &MetaHead::refreshRateChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(const QString & )>(_a, &MetaHead::relativeToChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(double )>(_a, &MetaHead::scaleChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(const QString & )>(_a, &MetaHead::serialChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(quint16 )>(_a, &MetaHead::transformChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(const QString & )>(_a, &MetaHead::verticalAnchorChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(int )>(_a, &MetaHead::widthChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(int )>(_a, &MetaHead::xChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (MetaHead::*)(int )>(_a, &MetaHead::yChanged, 24))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< bd::Outputs::OutputModeInfo >(); break;
        case 8:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< bd::Outputs::OutputModesMap >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->adaptiveSync(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->builtIn(); break;
        case 2: *reinterpret_cast<bd::Outputs::OutputModeInfo*>(_v) = _t->currentMode(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->description(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->enabled(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->height(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->horizontalAnchor(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->make(); break;
        case 8: *reinterpret_cast<bd::Outputs::OutputModesMap*>(_v) = _t->modes(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->mirrorOf(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->model(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->primary(); break;
        case 13: *reinterpret_cast<qulonglong*>(_v) = _t->refreshRate(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->relativeTo(); break;
        case 15: *reinterpret_cast<double*>(_v) = _t->scale(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->serial(); break;
        case 17: *reinterpret_cast<quint16*>(_v) = _t->transform(); break;
        case 18: *reinterpret_cast<QString*>(_v) = _t->verticalAnchor(); break;
        case 19: *reinterpret_cast<int*>(_v) = _t->width(); break;
        case 20: *reinterpret_cast<int*>(_v) = _t->x(); break;
        case 21: *reinterpret_cast<int*>(_v) = _t->y(); break;
        default: break;
        }
    }
}

const QMetaObject *bd::Outputs::Wlr::MetaHead::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *bd::Outputs::Wlr::MetaHead::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd7Outputs3Wlr8MetaHeadE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int bd::Outputs::Wlr::MetaHead::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void bd::Outputs::Wlr::MetaHead::headAvailable()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void bd::Outputs::Wlr::MetaHead::headNoLongerAvailable()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void bd::Outputs::Wlr::MetaHead::stateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void bd::Outputs::Wlr::MetaHead::adaptiveSyncChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void bd::Outputs::Wlr::MetaHead::currentModeChanged(const bd::Outputs::OutputModeInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void bd::Outputs::Wlr::MetaHead::descriptionChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void bd::Outputs::Wlr::MetaHead::enabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void bd::Outputs::Wlr::MetaHead::heightChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void bd::Outputs::Wlr::MetaHead::horizontalAnchorChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void bd::Outputs::Wlr::MetaHead::makeChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void bd::Outputs::Wlr::MetaHead::mirrorOfChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void bd::Outputs::Wlr::MetaHead::modelChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void bd::Outputs::Wlr::MetaHead::modesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void bd::Outputs::Wlr::MetaHead::nameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void bd::Outputs::Wlr::MetaHead::positionChanged(const QPoint & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void bd::Outputs::Wlr::MetaHead::primaryChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void bd::Outputs::Wlr::MetaHead::refreshRateChanged(qulonglong _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void bd::Outputs::Wlr::MetaHead::relativeToChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void bd::Outputs::Wlr::MetaHead::scaleChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void bd::Outputs::Wlr::MetaHead::serialChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void bd::Outputs::Wlr::MetaHead::transformChanged(quint16 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void bd::Outputs::Wlr::MetaHead::verticalAnchorChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void bd::Outputs::Wlr::MetaHead::widthChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void bd::Outputs::Wlr::MetaHead::xChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void bd::Outputs::Wlr::MetaHead::yChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1);
}
QT_WARNING_POP
