/****************************************************************************
** Meta object code from reading C++ file 'group.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/config/outputs/group.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'group.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN2bd6Config7Outputs5GroupE_t {};
} // unnamed namespace

template <> constexpr inline auto bd::Config::Outputs::Group::qt_create_metaobjectdata<qt_meta_tag_ZN2bd6Config7Outputs5GroupE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "bd::Config::Outputs::Group",
        "nameChanged",
        "",
        "name",
        "storedPrimaryOutputIdentifierChanged",
        "storedPrimaryOutputIdentifier",
        "storedIdentifiersChanged",
        "storedIdentifiers",
        "outputConfigsChanged",
        "QList<QSharedPointer<Output>>",
        "outputConfigs",
        "preferredChanged",
        "preferred"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'storedPrimaryOutputIdentifierChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'storedIdentifiersChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 7 },
        }}),
        // Signal 'outputConfigsChanged'
        QtMocHelpers::SignalData<void(const QList<QSharedPointer<Output>> &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'preferredChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'name'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'preferred'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'storedPrimaryOutputIdentifier'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'storedIdentifiers'
        QtMocHelpers::PropertyData<QList<QString>>(7, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'outputConfigs'
        QtMocHelpers::PropertyData<QList<QSharedPointer<Output>>>(10, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Group, qt_meta_tag_ZN2bd6Config7Outputs5GroupE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject bd::Config::Outputs::Group::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd6Config7Outputs5GroupE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd6Config7Outputs5GroupE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN2bd6Config7Outputs5GroupE_t>.metaTypes,
    nullptr
} };

void bd::Config::Outputs::Group::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Group *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->nameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->storedPrimaryOutputIdentifierChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->storedIdentifiersChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 3: _t->outputConfigsChanged((*reinterpret_cast<std::add_pointer_t<QList<QSharedPointer<Output>>>>(_a[1]))); break;
        case 4: _t->preferredChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QSharedPointer<Output>> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Group::*)(const QString & )>(_a, &Group::nameChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Group::*)(const QString & )>(_a, &Group::storedPrimaryOutputIdentifierChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Group::*)(const QStringList & )>(_a, &Group::storedIdentifiersChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Group::*)(const QList<QSharedPointer<Output>> & )>(_a, &Group::outputConfigsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Group::*)(bool )>(_a, &Group::preferredChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QSharedPointer<Output>> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->preferred(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->storedPrimaryOutputIdentifier(); break;
        case 3: *reinterpret_cast<QList<QString>*>(_v) = _t->storedIdentifiers(); break;
        case 4: *reinterpret_cast<QList<QSharedPointer<Output>>*>(_v) = _t->outputConfigs(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setPreferred(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setStoredPrimaryOutputIdentifier(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setStoredIdentifiers(*reinterpret_cast<QList<QString>*>(_v)); break;
        case 4: _t->setOutputConfigs(*reinterpret_cast<QList<QSharedPointer<Output>>*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *bd::Config::Outputs::Group::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *bd::Config::Outputs::Group::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd6Config7Outputs5GroupE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int bd::Config::Outputs::Group::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void bd::Config::Outputs::Group::nameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void bd::Config::Outputs::Group::storedPrimaryOutputIdentifierChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void bd::Config::Outputs::Group::storedIdentifiersChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void bd::Config::Outputs::Group::outputConfigsChanged(const QList<QSharedPointer<Output>> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void bd::Config::Outputs::Group::preferredChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
