/****************************************************************************
** Meta object code from reading C++ file 'state.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/outputs/state.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'state.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN2bd7Outputs5StateE_t {};
} // unnamed namespace

template <> constexpr inline auto bd::Outputs::State::qt_create_metaobjectdata<qt_meta_tag_ZN2bd7Outputs5StateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "bd::Outputs::State",
        "D-Bus Interface",
        "org.buddiesofbudgie.Services.Outputs",
        "ready",
        "",
        "done",
        "orchestratorInitFailed",
        "error",
        "availableOutputsChanged",
        "globalRectChanged",
        "primaryOutputChanged",
        "primaryOutputRectChanged",
        "outputManagerDone",
        "onHeadAdded",
        "QSharedPointer<Wlr::MetaHead>",
        "head",
        "onHeadRemoved",
        "checkAndEmitSignals",
        "availableOutputs",
        "globalRect",
        "QVariantMap",
        "primaryOutput",
        "primaryOutputRect"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ready'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'done'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'orchestratorInitFailed'
        QtMocHelpers::SignalData<void(QString)>(6, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'availableOutputsChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'globalRectChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'primaryOutputChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'primaryOutputRectChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'outputManagerDone'
        QtMocHelpers::SlotData<void()>(12, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onHeadAdded'
        QtMocHelpers::SlotData<void(QSharedPointer<Wlr::MetaHead>)>(13, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'onHeadRemoved'
        QtMocHelpers::SlotData<void(QSharedPointer<Wlr::MetaHead>)>(16, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'checkAndEmitSignals'
        QtMocHelpers::SlotData<void()>(17, 4, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'availableOutputs'
        QtMocHelpers::PropertyData<QStringList>(18, QMetaType::QStringList, QMC::DefaultPropertyFlags, 3),
        // property 'globalRect'
        QtMocHelpers::PropertyData<QVariantMap>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'primaryOutput'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'primaryOutputRect'
        QtMocHelpers::PropertyData<QVariantMap>(22, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<State, qt_meta_tag_ZN2bd7Outputs5StateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject bd::Outputs::State::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd7Outputs5StateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd7Outputs5StateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN2bd7Outputs5StateE_t>.metaTypes,
    nullptr
} };

void bd::Outputs::State::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<State *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ready(); break;
        case 1: _t->done(); break;
        case 2: _t->orchestratorInitFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->availableOutputsChanged(); break;
        case 4: _t->globalRectChanged(); break;
        case 5: _t->primaryOutputChanged(); break;
        case 6: _t->primaryOutputRectChanged(); break;
        case 7: _t->outputManagerDone(); break;
        case 8: _t->onHeadAdded((*reinterpret_cast<std::add_pointer_t<QSharedPointer<Wlr::MetaHead>>>(_a[1]))); break;
        case 9: _t->onHeadRemoved((*reinterpret_cast<std::add_pointer_t<QSharedPointer<Wlr::MetaHead>>>(_a[1]))); break;
        case 10: _t->checkAndEmitSignals(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (State::*)()>(_a, &State::ready, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (State::*)()>(_a, &State::done, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (State::*)(QString )>(_a, &State::orchestratorInitFailed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (State::*)()>(_a, &State::availableOutputsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (State::*)()>(_a, &State::globalRectChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (State::*)()>(_a, &State::primaryOutputChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (State::*)()>(_a, &State::primaryOutputRectChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QStringList*>(_v) = _t->availableOutputs(); break;
        case 1: *reinterpret_cast<QVariantMap*>(_v) = _t->globalRect(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->primaryOutput(); break;
        case 3: *reinterpret_cast<QVariantMap*>(_v) = _t->primaryOutputRect(); break;
        default: break;
        }
    }
}

const QMetaObject *bd::Outputs::State::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *bd::Outputs::State::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2bd7Outputs5StateE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int bd::Outputs::State::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void bd::Outputs::State::ready()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void bd::Outputs::State::done()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void bd::Outputs::State::orchestratorInitFailed(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void bd::Outputs::State::availableOutputsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void bd::Outputs::State::globalRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void bd::Outputs::State::primaryOutputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void bd::Outputs::State::primaryOutputRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
